/*
 GBxCart RW - Console Interface Flasher
 Version: 1.25
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 26/08/2017
 Last Modified: 2/12/2019
 License: GPL
 
 This program allows you to write ROMs to Flash Carts that are supported.
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>

#ifdef _WIN32
#include <Windows.h>
#else
#include <unistd.h>
#endif

#include "setup.h" // See defines, variables, constants, functions here

int main(int argc, char **argv) {
	
	printf("GBxCart RW Flasher v1.25 by insideGadgets\n");
	printf("#########################################\n");
	
	// Check arguments
read_config();
	
	// Open COM port
	if (com_test_port() == 0) {
		printf("Device not connected and couldn't be auto detected\n");
		read_one_letter();
		return 1;
	}
	printf("Connected on COM port: %i\n", cport_nr+1);
	
	// Break out of any existing functions on ATmega
	set_mode('0');
	
	// Get cartridge mode - Gameboy or Gameboy Advance
	cartridgeMode = request_value(CART_MODE);
	
	// Get firmware version
	gbxcartFirmwareVersion = request_value(READ_FIRMWARE_VERSION);
	
	// Get PCB version
	gbxcartPcbVersion = request_value(READ_PCB_VERSION);
	
	if (gbxcartPcbVersion == PCB_1_0) {
		printf("\nPCB v1.0 is not supported for this function.");
		read_one_letter();
		return 1;
	}
	
	if (gbxcartFirmwareVersion <= 8) {
		printf("Firmware R9 or higher is required for this functionality.\n");
		read_one_letter();
		return 1;
	}
	
	// PCB v1.3 - Set 5V
	if (gbxcartPcbVersion == PCB_1_3) {
		set_mode(VOLTAGE_5V);
		delay_ms(500);
	}
	
	// Flash Setup
	set_mode(GB_CART_MODE); // Gameboy mode
	gb_flash_pin_setup(WE_AS_AUDIO_PIN); // Audio pin
	gb_flash_program_setup(GB_FLASH_PROGRAM_5555);// Flash program byte method
	gb_check_change_flash_id(GB_FLASH_PROGRAM_5555);
	
	
	printf("\nChip erase? [y/n]: ");
	char optionString[5];
	fgets(optionString, 5, stdin);
	if (optionString[0] == 'y') {
		
		printf("\nErasing");
		
		// Write ROM
		set_number(currAddr, SET_START_ADDRESS);
		delay_ms(5);
		
		gb_flash_write_address_byte(0x5555, 0xAA);
		gb_flash_write_address_byte(0x2AAA, 0x55);
		gb_flash_write_address_byte(0x5555, 0x80);
		gb_flash_write_address_byte(0x5555, 0xAA);
		gb_flash_write_address_byte(0x2AAA, 0x55);
		gb_flash_write_address_byte(0x5555, 0x10);
		
		// Wait for first byte to be 0xFF
		wait_for_flash_chip_erase_ff(1);
	}

	printf("\n");
	
	return 0;
}